@@ -1,9 +1,9 @@
 {-# LANGUAGE Trustworthy #-}
 {-|
 Module      : Github.Data.Webhooks.Events
-Copyright   : (c) ONROCK, 2018
+Copyright   : (c) Cuedo Control Engineering 2017-2022
 License     : MIT
-Maintainer  : Kyle Van Berendonck <foss@onrock.online>
+Maintainer  : Kyle Van Berendonck <foss@cuedo.com.au>
 
 This module contains types that represent GitHub webhook's events.
 -}
@@ -11,6 +11,12 @@ module GitHub.Data.Webhooks.Events
     ( EventHasSender(..)
     , EventHasRepo(..)
       --
+    , CheckSuiteEventAction(..)
+    , CheckSuiteEvent(..)
+      --
+    , CheckRunEventAction(..)
+    , CheckRunEvent(..)
+      --
     , CommitCommentEvent(..)
     , CommitCommentEventAction(..)
       --
@@ -45,6 +51,9 @@ module GitHub.Data.Webhooks.Events
     , LabelEvent(..)
     , LabelEventAction(..)
       --
+    , MarketplacePurchaseEvent(..)
+    , MarketplacePurchaseEventAction(..)
+      --
     , MemberEvent(..)
     , MemberEventAction(..)
       --
@@ -103,10 +112,14 @@ module GitHub.Data.Webhooks.Events
     ) where
 
 import           Data.Aeson               (FromJSON(..), withObject, withText, (.:), (.:?), (.!=))
+import           Control.Applicative      ((<*>), pure)
 import           Control.DeepSeq          (NFData(..))
 import           Control.DeepSeq.Generics (genericRnf)
 import           Data.Data                (Data, Typeable)
+import           Data.Functor             ((<$>))
+import           Data.Monoid              (mempty)
 import           Data.Time                (UTCTime)
+import           Data.Time.LocalTime      (zonedTimeToUTC)
 import           Data.Text                (Text)
 import           Data.Vector              (Vector)
 import           GHC.Generics             (Generic)
@@ -123,6 +136,88 @@ class EventHasRepo eventKind where
     -- | Provides the repository context of a Webhook event.
     repoForEvent :: eventKind -> HookRepository
 
+-- | Represents the "action" field in the
+-- 'CheckSuiteEvent' payload.
+data CheckSuiteEventAction
+    -- | Decodes from "completed"
+    = CheckSuiteEventActionCompleted
+    -- | Decodes from "requested"
+    | CheckSuiteEventActionRequested
+    -- | Decodes from "rerequested"
+    | CheckSuiteEventActionRerequested
+    -- | The result of decoding an unknown check suite event action type
+    | CheckSuiteEventActionOther !Text
+    deriving (Eq, Ord, Show, Generic, Typeable, Data)
+
+instance NFData CheckSuiteEventAction where rnf = genericRnf
+
+instance FromJSON CheckSuiteEventAction where
+  parseJSON = withText "Check suite event action" $ \t ->
+      case t of
+          "completed"          -> pure CheckSuiteEventActionCompleted
+          "requested"          -> pure CheckSuiteEventActionRequested
+          "rerequested"        -> pure CheckSuiteEventActionRerequested
+          _                    -> pure (CheckSuiteEventActionOther t)
+
+-- | Triggered when a check suite is completed, requested, or rerequested.
+-- See <https://developer.github.com/v3/activity/events/types/#checksuiteevent>.
+data CheckSuiteEvent = CheckSuiteEvent
+    { evCheckSuiteAction              :: !CheckSuiteEventAction
+    , evCheckSuiteCheckSuite          :: !HookCheckSuite
+    , evCheckSuiteRepository          :: !HookRepository
+    , evCheckSuiteOrganization        :: !(Maybe HookOrganization)
+    , evCheckSuiteSender              :: !HookUser
+    , evCheckSuiteInstallation        :: !(Maybe HookChecksInstallation)
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance EventHasSender CheckSuiteEvent where senderOfEvent = evCheckSuiteSender
+instance EventHasRepo CheckSuiteEvent where repoForEvent = evCheckSuiteRepository
+instance NFData CheckSuiteEvent where rnf = genericRnf
+
+-- | Represents the "action" field in the
+-- 'CheckRunEvent' payload.
+data CheckRunEventAction
+    -- | Decodes from "created"
+    = CheckRunEventActionCreated
+    -- | Decodes from "completed"
+    | CheckRunEventActionCompleted
+    -- | Decodes from "rerequested"
+    | CheckRunEventActionRerequested
+    -- | Decodes from "requested_action"
+    | CheckRunEventActionRequestedAction
+    -- | The result of decoding an unknown check run event action type
+    | CheckRunEventActionOther !Text
+    deriving (Eq, Ord, Show, Generic, Typeable, Data)
+
+instance NFData CheckRunEventAction where rnf = genericRnf
+
+instance FromJSON CheckRunEventAction where
+  parseJSON = withText "Check run event action" $ \t ->
+      case t of
+          "created"            -> pure CheckRunEventActionCreated
+          "completed"          -> pure CheckRunEventActionCompleted
+          "rerequested"        -> pure CheckRunEventActionRerequested
+          "requested_action"   -> pure CheckRunEventActionRequestedAction
+          _                    -> pure (CheckRunEventActionOther t)
+
+-- | Triggered when a check run is created, rerequested, completed, or has a requested_action.
+-- See <https://developer.github.com/v3/activity/events/types/#checkrunevent>.
+data CheckRunEvent = CheckRunEvent
+    { evCheckRunAction              :: !CheckRunEventAction
+    , evCheckRunCheckRun            :: !HookCheckRun
+    , evCheckRunRequestedAction     :: !(Maybe HookCheckRunRequestedAction)
+    , evCheckRunRepository          :: !HookRepository
+    , evCheckRunOrganization        :: !(Maybe HookOrganization)
+    , evCheckRunSender              :: !HookUser
+    , evCheckRunInstallation        :: !(Maybe HookChecksInstallation)
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance EventHasSender CheckRunEvent where senderOfEvent = evCheckRunSender
+instance EventHasRepo CheckRunEvent where repoForEvent = evCheckRunRepository
+instance NFData CheckRunEvent where rnf = genericRnf
+
 -- | Represents the "action" field in the
 -- 'CommitCommentEvent' payload.
 data CommitCommentEventAction
@@ -278,6 +373,12 @@ data InstallationEventAction
   = InstallationCreatedAction
   -- | Decodes from "deleted"
   | InstallationDeletedAction
+  -- | Decodes from "suspend"
+  | InstallationSuspendAction
+  -- | Decodes from "unsuspend"
+  | InstallationUnsuspendAction
+  -- | Decodes from "new_permissions_accepted"
+  | InstallationNewPermissionsAcceptedAction
   -- | The result of decoding an unknown installation event action type
   | InstallationActionOther !Text
   deriving (Eq, Ord, Show, Generic, Typeable, Data)
@@ -287,9 +388,12 @@ instance NFData InstallationEventAction where rnf = genericRnf
 instance FromJSON InstallationEventAction where
   parseJSON = withText "Installation event action" $ \t ->
     case t of
-        "created"       -> pure InstallationCreatedAction
-        "deleted"       -> pure InstallationDeletedAction
-        _               -> pure (InstallationActionOther t)
+        "created"                  -> pure InstallationCreatedAction
+        "deleted"                  -> pure InstallationDeletedAction
+        "suspend"                  -> pure InstallationSuspendAction
+        "unsuspend"                -> pure InstallationUnsuspendAction
+        "new_permissions_accepted" -> pure InstallationNewPermissionsAcceptedAction
+        _                          -> pure (InstallationActionOther t)
 
 -- | Triggered when a GitHub App has been installed or uninstalled.
 -- See <https://developer.github.com/v3/activity/events/types/#installationevent>.
@@ -306,8 +410,8 @@ instance NFData InstallationEvent where rnf = genericRnf
 
 
 data InstallationRepoEventAction
-  -- | Decodes from "created"
-  = InstallationRepoCreatedAction
+  -- | Decodes from "added"
+  = InstallationRepoAddedAction
   -- | Decodes from "removed"
   | InstallationRepoRemovedAction
   -- | The result of decoding an unknown installation repo event action type
@@ -319,7 +423,7 @@ instance NFData InstallationRepoEventAction where rnf = genericRnf
 instance FromJSON InstallationRepoEventAction where
   parseJSON = withText "Installation repo event action" $ \t ->
     case t of
-        "created"       -> pure InstallationRepoCreatedAction
+        "added"         -> pure InstallationRepoAddedAction
         "removed"       -> pure InstallationRepoRemovedAction
         _               -> pure (InstallationRepoActionOther t)
 
@@ -472,6 +576,48 @@ instance EventHasRepo LabelEvent where repoForEvent = evLabelEventRepo
 instance NFData LabelEvent where rnf = genericRnf
 
 
+data MarketplacePurchaseEventAction
+  -- | Decodes from "purchased"
+  = MarketplacePurchasePurchasedAction
+  -- | Decodes from "cancelled"
+  | MarketplacePurchaseCancelledAction
+  -- | Decodes from "pending_change"
+  | MarketplacePurchasePendingChangeAction
+  -- | Decodes from "pending_change_cancelled"
+  | MarketplacePurchasePendingChangeCancelledAction
+  -- | Decodes from "changed"
+  | MarketplacePurchaseChangedAction
+  -- | The result of decoding an unknown marketplace purchase event action type
+  | MarketplacePurchaseActionOther !Text
+  deriving (Eq, Ord, Show, Generic, Typeable, Data)
+
+instance NFData MarketplacePurchaseEventAction where rnf = genericRnf
+
+instance FromJSON MarketplacePurchaseEventAction where
+  parseJSON = withText "Marketplace purchase event action" $ \t ->
+    case t of
+        "purchased"                -> pure MarketplacePurchasePurchasedAction
+        "cancelled"                -> pure MarketplacePurchaseCancelledAction
+        "pending_change"           -> pure MarketplacePurchasePendingChangeAction
+        "pending_change_cancelled" -> pure MarketplacePurchasePendingChangeCancelledAction
+        "changed"                  -> pure MarketplacePurchaseChangedAction
+        _                          -> pure (MarketplacePurchaseActionOther t)
+
+-- | A GitHub Marketplace app receives information about changes to a user's plan from the Marketplace purchase event webhook. A Marketplace purchase event is triggered when a user purchases, cancels, or changes their payment plan.
+-- See <https://docs.github.com/en/developers/github-marketplace/using-the-github-marketplace-api-in-your-app/webhook-events-for-the-github-marketplace-api#github-marketplace-purchase-webhook-payload>.
+data MarketplacePurchaseEvent = MarketplacePurchaseEvent
+    { evMarketplacePurchaseAction        :: !MarketplacePurchaseEventAction
+    , evMarketplacePurchaseEffectiveDate :: !UTCTime
+    , evMarketplacePurchaseSender        :: !HookUser
+    , evMarketplacePurchaseNew           :: !HookMarketplacePurchase
+    , evMarketplacePurchasePrevious      :: !(Maybe HookMarketplacePurchase)
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance EventHasSender MarketplacePurchaseEvent where senderOfEvent = evMarketplacePurchaseSender
+instance NFData MarketplacePurchaseEvent where rnf = genericRnf
+
+
 data MemberEventAction
   -- | Decodes from "added"
   = MemberAddedAction
@@ -613,8 +759,8 @@ instance FromJSON OrganizationEventAction where
 -- See <https://developer.github.com/v3/activity/events/types/#organizationevent>.
 data OrganizationEvent = OrganizationEvent
     { evOrganizationAction      :: !OrganizationEventAction
-    , evOrganizationInvitation  :: !HookOrganizationInvitation
-    , evOrganizationMembership  :: !HookOrganizationMembership
+    , evOrganizationInvitation  :: !(Maybe HookOrganizationInvitation)
+    , evOrganizationMembership  :: !(Maybe HookOrganizationMembership)
     , evOrganizationOrg         :: !HookOrganization
     , evOrganizationSender      :: !HookUser
     }
@@ -1191,6 +1337,25 @@ instance NFData WatchEvent where rnf = genericRnf
 
 -- Aeson Instances
 
+instance FromJSON CheckSuiteEvent where
+    parseJSON = withObject "CheckSuiteEvent" $ \o -> CheckSuiteEvent
+        <$> o .: "action"
+        <*> o .: "check_suite"
+        <*> o .: "repository"
+        <*> o .:? "organization"
+        <*> o .: "sender"
+        <*> o .:? "installation"
+
+instance FromJSON CheckRunEvent where
+    parseJSON = withObject "CheckRunEvent" $ \o -> CheckRunEvent
+        <$> o .: "action"
+        <*> o .: "check_run"
+        <*> o .:? "requested_action"
+        <*> o .: "repository"
+        <*> o .:? "organization"
+        <*> o .: "sender"
+        <*> o .:? "installation"
+
 instance FromJSON CommitCommentEvent where
     parseJSON = withObject "CommitCommentEvent" $ \o -> CommitCommentEvent
         <$> o .: "action"
@@ -1280,6 +1445,14 @@ instance FromJSON LabelEvent where
         <*> o .:? "organization"
         <*> o .: "sender"
 
+instance FromJSON MarketplacePurchaseEvent where
+    parseJSON = withObject "MarketplacePurchaseEvent" $ \o -> MarketplacePurchaseEvent
+        <$> o .: "action"
+        <*> (zonedTimeToUTC <$> o .: "effective_date")
+        <*> o .: "sender"
+        <*> o .: "marketplace_purchase"
+        <*> o .:? "previous_marketplace_purchase"
+
 instance FromJSON MemberEvent where
     parseJSON = withObject "MemberEvent" $ \o -> MemberEvent
         <$> o .: "action"
@@ -1307,8 +1480,8 @@ instance FromJSON MilestoneEvent where
 instance FromJSON OrganizationEvent where
     parseJSON = withObject "OrganizationEvent" $ \o -> OrganizationEvent
         <$> o .: "action"
-        <*> o .: "invitation"
-        <*> o .: "membership"
+        <*> o .:? "invitation"
+        <*> o .:? "membership"
         <*> o .: "organization"
         <*> o .: "sender"
 
