@@ -1,9 +1,9 @@
 {-# LANGUAGE Trustworthy #-}
 {-|
 Module      : Github.Data.Webhooks.Payload
-Copyright   : (c) ONROCK, 2018
+Copyright   : (c) Cuedo Control Engineering 2017-2022
 License     : MIT
-Maintainer  : Kyle Van Berendonck <foss@onrock.online>
+Maintainer  : Kyle Van Berendonck <foss@cuedo.com.au>
 
 This module contains types that represent GitHub webhook's payload contents.
 -}
@@ -23,6 +23,11 @@ module GitHub.Data.Webhooks.Payload
     , HookOrganizationInvitation(..)
     , HookOrganizationMembership(..)
     , HookTeam(..)
+    , HookMarketplaceAccount(..)
+    , HookMarketplaceBillingCycle(..)
+    , HookMarketplacePlan(..)
+    , HookMarketplacePlanPriceModel(..)
+    , HookMarketplacePurchase(..)
     , HookMilestone(..)
     , HookMembership(..)
     , HookProject(..)
@@ -30,6 +35,19 @@ module GitHub.Data.Webhooks.Payload
     , HookProjectColumn(..)
     , HookIssueLabels(..)
     , HookCommit(..)
+    , HookCheckSuiteStatus(..)
+    , HookCheckSuiteConclusion(..)
+    , HookCheckSuite(..)
+    , HookCheckSuiteCommit(..)
+    , HookCheckRunStatus(..)
+    , HookCheckRunConclusion(..)
+    , HookCheckRun(..)
+    , HookCheckRunOutput(..)
+    , HookCheckRunRequestedAction(..)
+    , HookChecksInstallation(..)
+    , HookChecksPullRequest(..)
+    , HookChecksPullRequestRepository(..)
+    , HookChecksPullRequestTarget(..)
     , HookRelease(..)
     , HookPullRequest(..)
     , PullRequestTarget(..)
@@ -47,9 +65,11 @@ module GitHub.Data.Webhooks.Payload
 import           Data.Aeson               (FromJSON(..), withObject, withText, (.!=), (.:), (.:?))
 import           Control.DeepSeq          (NFData (..))
 import           Control.DeepSeq.Generics (genericRnf)
-import           Control.Applicative      ((<|>))
+import           Control.Applicative      ((<|>), (<*>), pure)
 import           Data.Data                (Data, Typeable)
+import           Data.Functor             ((<$>))
 import           Data.Time                (UTCTime)
+import           Data.Time.LocalTime      (zonedTimeToUTC)
 import           Data.Time.Clock.POSIX    (posixSecondsToUTCTime)
 import           Data.Text                (Text)
 import qualified Data.Text                as T
@@ -105,6 +125,7 @@ data HookIssue = HookIssue
     , whIssueEventsUrl          :: !URL
     , whIssueHtmlUrl            :: !URL
     , whIssueId                 :: !Int
+    , whIssueNodeId             :: !Text
     , whIssueNumber             :: !Int
     , whIssueTitle              :: !Text
     , whIssueUser               :: !HookUser
@@ -126,6 +147,7 @@ instance NFData HookIssue where rnf = genericRnf
 -- | Represents the "repository" field in all types of payload.
 data HookRepository = HookRepository
     { whRepoId                  :: !Int
+    , whRepoNodeId              :: !Text
     , whRepoName                :: !Text
     , whRepoFullName            :: !Text
     , whRepoOwner               :: !(Either HookSimpleUser HookUser)
@@ -198,6 +220,7 @@ instance NFData HookRepository where rnf = genericRnf
 -- field in the 'InstallationRepositoriesEvent' payload.
 data HookRepositorySimple = HookRepositorySimple
   { whSimplRepoId               :: !Int
+  , whSimplRepoNodeId           :: !Text
   , whSimplRepoName             :: !Text
   , whSimplRepoFullName         :: !Text
   , whSimplRepoIsPrivate        :: !Bool
@@ -208,7 +231,8 @@ instance NFData HookRepositorySimple where rnf = genericRnf
 
 -- | Represents the "label" field in the 'LabelEvent' payload.
 data HookRepositoryLabel = HookRepositoryLabel
-    { whRepoLabelUrl            :: !URL
+    { whRepoLabelNodeId         :: !(Maybe Text)
+    , whRepoLabelUrl            :: !URL
     , whRepoLabelName           :: !Text
     , whRepoLabelColor          :: !Text
     }
@@ -220,6 +244,7 @@ instance NFData HookRepositoryLabel where rnf = genericRnf
 data HookUser = HookUser
     { whUserLogin               :: !Text
     , whUserId                  :: !Int
+    , whUserNodeId              :: !Text
     , whUserAvatarUrl           :: !URL
     , whUserGravatarId          :: !URL
     , whUserUrl                 :: !URL
@@ -254,6 +279,7 @@ instance NFData HookSimpleUser where rnf = genericRnf
 data HookOrganization = HookOrganization
     { whOrgLogin                :: !Text
     , whOrgId                   :: !Int
+    , whOrgNodeId               :: !Text
     , whOrgUrl                  :: !URL
     , whOrgReposUrl             :: !URL
     , whOrgEventsUrl            :: !URL
@@ -271,6 +297,7 @@ instance NFData HookOrganization where rnf = genericRnf
 -- | Represents the "invitation" field in the 'OrganizationEvent' payload.
 data HookOrganizationInvitation = HookOrganizationInvitation
     { whOrgInvitationId         :: !Int
+    , whOrgInvitationNodeId     :: !Text
     , whOrgInvitationLogin      :: !Text
     , whOrgInvitationEmail      :: !(Maybe Text)
     , whOrgInvitationRole       :: !Text
@@ -296,6 +323,7 @@ instance NFData HookOrganizationMembership where rnf = genericRnf
 data HookTeam = HookTeam
     { whTeamName                :: !Text
     , whTeamId                  :: !Int
+    , whTeamNodeId              :: !Text
     , whTeamSlug                :: !Text
     , whTeamPermission          :: !Text
     , whTeamUrl                 :: !URL
@@ -306,6 +334,90 @@ data HookTeam = HookTeam
 
 instance NFData HookTeam where rnf = genericRnf
 
+-- | Represents the "billing_cycle" field in the
+-- 'HookMarketplacePurchase' payload.
+data HookMarketplaceBillingCycle
+    -- | Decodes from "yearly"
+    = HookMarketplaceBillingCycleYearly
+    -- | Decodes from "monthly".
+    | HookMarketplaceBillingCycleMonthly
+    -- | The result of decoding an unknown marketplace purchase billing cycle type
+    | HookMarketplaceBillingCycleOther !Text
+    deriving (Eq, Ord, Show, Generic, Typeable, Data)
+
+instance NFData HookMarketplaceBillingCycle where rnf = genericRnf
+
+instance FromJSON HookMarketplaceBillingCycle where
+  parseJSON = withText "Hook marketplace billing cycle" $ \t ->
+      case t of
+          "yearly"          -> pure HookMarketplaceBillingCycleYearly
+          "monthly"         -> pure HookMarketplaceBillingCycleMonthly
+          _                 -> pure (HookMarketplaceBillingCycleOther t)
+
+-- | Represents the "marketplace_purchase" field in the 'MarketplacePurchaseEvent' payload.
+data HookMarketplacePurchase = HookMarketplacePurchase
+    { whMarketplacePurchaseAccount         :: !HookMarketplaceAccount
+    , whMarketplacePurchaseBillingCycle    :: !(Maybe HookMarketplaceBillingCycle)
+    , whMarketplacePurchaseUnitCount       :: !Int
+    , whMarketplacePurchaseOnFreeTrial     :: !Bool
+    , whMarketplacePurchaseFreeTrialEndsOn :: !(Maybe UTCTime)
+    , whMarketplacePurchaseNextBillingDate :: !(Maybe UTCTime)
+    , whMarketplacePurchasePlan            :: !HookMarketplacePlan
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookMarketplacePurchase where rnf = genericRnf
+
+-- | Represents the "account" field in the 'HookMarketplacePurchase' payload.
+data HookMarketplaceAccount = HookMarketplaceAccount
+    { whMarketplaceAccountType                     :: !OwnerType
+    , whMarketplaceAccountId                       :: !Int
+    , whMarketplaceAccountNodeId                   :: !Text
+    , whMarketplaceAccountLogin                    :: !Text
+    , whMarketplaceAccountOrganizationBillingEmail :: !(Maybe Text)
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookMarketplaceAccount where rnf = genericRnf
+
+-- | Represents the "plan" field in the 'HookMarketplacePurchase' payload.
+data HookMarketplacePlan = HookMarketplacePlan
+    { whMarketplacePlanId                  :: !Int
+    , whMarketplacePlanName                :: !Text
+    , whMarketplacePlanDescription         :: !Text
+    , whMarketplacePlanMonthlyPriceInCents :: !Int
+    , whMarketplacePlanYearlyPriceInCents  :: !Int
+    , whMarketplacePlanPriceModel          :: !HookMarketplacePlanPriceModel
+    , whMarketplacePlanHasFreeTrial        :: !Bool
+    , whMarketplacePlanUnitName            :: !(Maybe Text)
+    , whMarketplacePlanBullet              :: !(Vector Text)
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookMarketplacePlan where rnf = genericRnf
+
+-- | Represents the "price_model" field in the
+-- 'HookMarketplacePlan' payload.
+data HookMarketplacePlanPriceModel
+    -- | Decodes from "flat-rate"
+    = HookMarketplacePlanPriceModelFlatRate
+    -- | Decodes from "per-unit".
+    | HookMarketplacePlanPriceModelPerUnit
+    -- | Decodes from "free".
+    | HookMarketplacePlanPriceModelFree
+    -- | The result of decoding an unknown marketplace plan price model
+    | HookMarketplacePlanPriceModelOther !Text
+    deriving (Eq, Ord, Show, Generic, Typeable, Data)
+
+instance NFData HookMarketplacePlanPriceModel where rnf = genericRnf
+
+instance FromJSON HookMarketplacePlanPriceModel where
+  parseJSON = withText "Hook marketplace plan price model" $ \t ->
+      case t of
+          "flat-rate"       -> pure HookMarketplacePlanPriceModelFlatRate
+          "per-unit"        -> pure HookMarketplacePlanPriceModelPerUnit
+          "free"            -> pure HookMarketplacePlanPriceModelFree
+          _                 -> pure (HookMarketplacePlanPriceModelOther t)
 
 type MilestoneState = Text
 
@@ -315,6 +427,7 @@ data HookMilestone = HookMilestone
     , whMilestoneHtmlUrl        :: !URL
     , whMilestoneLabelsUrl      :: !URL
     , whMilestoneId             :: !Int
+    , whMilestoneNodeId         :: !Text
     , whMilestoneNumber         :: !Int
     , whMilestoneTitle          :: !Text
     , whMilestoneDescription    :: !(Maybe Text)
@@ -356,6 +469,7 @@ data HookProject = HookProject
     , whProjectUrl              :: !URL
     , whProjectColumnsUrl       :: !URL
     , whProjectId               :: !Int
+    , whProjectNodeId           :: !Text
     , whProjectName             :: !Text
     , whProjectBody             :: !Text
     , whProjectNumber           :: !Int
@@ -374,6 +488,7 @@ data HookProjectCard = HookProjectCard
     , whProjectCardColumnUrl    :: !URL
     , whProjectCardColumnId     :: !Int
     , whProjectCardId           :: !Int
+    , whProjectCardNodeId       :: !Text
     , whProjectCardNote         :: !(Maybe Text)
     , whProjectCardCreator      :: !HookUser
     , whProjectCardCreatedAt    :: !UTCTime
@@ -390,6 +505,7 @@ data HookProjectColumn = HookProjectColumn
     , whProjectColumnProjUrl    :: !URL
     , whProjectColumnCardsUrl   :: !URL
     , whProjectColumnId         :: !Int
+    , whProjectColumnNodeId     :: !Text
     , whProjectColumnName       :: !Text
     , whProjectColumnCreatedAt  :: !UTCTime
     , whProjectColumnUpdatedAt  :: !UTCTime
@@ -402,6 +518,7 @@ instance NFData HookProjectColumn where rnf = genericRnf
 -- 'IssueCommentEvent' and 'IssueEvent' payloads.
 data HookIssueLabels = HookIssueLabels
     { whIssueLabelId            :: !(Maybe Int)   -- ^ Not always sent.
+    , whIssueLabelNodeId        :: !(Maybe Text)
     , whIssueLabelUrl           :: !URL
     , whIssueLabelName          :: !Text
     , whIssueLabelColor         :: !Text
@@ -411,9 +528,250 @@ data HookIssueLabels = HookIssueLabels
 
 instance NFData HookIssueLabels where rnf = genericRnf
 
+-- | Represents the "status" field in the
+-- 'HookCheckSuite' payload.
+data HookCheckSuiteStatus
+    -- | Decodes from "requested"
+    = HookCheckSuiteStatusRequested
+    -- | Decodes from "queued".
+    | HookCheckSuiteStatusQueued
+    -- | Decodes from "in_progress"
+    | HookCheckSuiteStatusInProgress
+    -- | Decodes from "completed"
+    | HookCheckSuiteStatusCompleted
+    -- | The result of decoding an unknown check suite status type
+    | HookCheckSuiteStatusOther !Text
+    deriving (Eq, Ord, Show, Generic, Typeable, Data)
 
--- FIXME: Missing nested metadata that provides commit description
--- FIXME: Missing property "parent" (no examples provided)
+instance NFData HookCheckSuiteStatus where rnf = genericRnf
+
+instance FromJSON HookCheckSuiteStatus where
+  parseJSON = withText "Hook check suite status" $ \t ->
+      case t of
+          "requested"          -> pure HookCheckSuiteStatusRequested
+          "queued"             -> pure HookCheckSuiteStatusQueued
+          "in_progress"        -> pure HookCheckSuiteStatusInProgress
+          "completed"          -> pure HookCheckSuiteStatusCompleted
+          _                    -> pure (HookCheckSuiteStatusOther t)
+
+-- | Represents the "conclusion" field in the
+-- 'HookCheckSuite' payload.
+data HookCheckSuiteConclusion
+    -- | Decodes from "success"
+    = HookCheckSuiteConclusionSuccess
+    -- | Decodes from "failure"
+    | HookCheckSuiteConclusionFailure
+    -- | Decodes from "neutral"
+    | HookCheckSuiteConclusionNeutral
+    -- | Decodes from "cancelled"
+    | HookCheckSuiteConclusionCancelled
+    -- | Decodes from "timed_out"
+    | HookCheckSuiteConclusionTimedOut
+    -- | Decodes from "action_required"
+    | HookCheckSuiteConclusionActionRequired
+    -- | Decodes from "stale"
+    | HookCheckSuiteConclusionStale
+    -- | The result of decoding an unknown check suite conclusion type
+    | HookCheckSuiteConclusionOther !Text
+    deriving (Eq, Ord, Show, Generic, Typeable, Data)
+
+instance NFData HookCheckSuiteConclusion where rnf = genericRnf
+
+instance FromJSON HookCheckSuiteConclusion where
+  parseJSON = withText "Hook check suite status" $ \t ->
+      case t of
+          "success"               -> pure HookCheckSuiteConclusionSuccess
+          "failure"               -> pure HookCheckSuiteConclusionFailure
+          "neutral"               -> pure HookCheckSuiteConclusionNeutral
+          "cancelled"             -> pure HookCheckSuiteConclusionCancelled
+          "timed_out"             -> pure HookCheckSuiteConclusionTimedOut
+          "action_required"       -> pure HookCheckSuiteConclusionActionRequired
+          "stale"                 -> pure HookCheckSuiteConclusionStale
+          _                       -> pure (HookCheckSuiteConclusionOther t)
+
+-- FIXME: Missing nested "app", there are examples, but no documentation.
+-- | Represents the "check_suite" field in the
+-- 'CheckSuiteEvent' payload.
+data HookCheckSuite = HookCheckSuite
+    { whCheckSuiteId                   :: !Int
+    , whCheckSuiteNodeId               :: !Text
+    , whCheckSuiteHeadBranch           :: !(Maybe Text) -- ^ The Checks API only looks for pushes in the repository where the check suite or check run were created. Pushes to a branch in a forked repository are not detected and return an empty pull_requests array and a null value for head_branch.
+    , whCheckSuiteHeadSha              :: !Text
+    , whCheckSuiteStatus               :: !HookCheckSuiteStatus
+    , whCheckSuiteConclusion           :: !(Maybe HookCheckSuiteConclusion)
+    , whCheckSuiteUrl                  :: !URL
+    , whCheckSuiteBeforeSha            :: !(Maybe Text)
+    , whCheckSuiteAfterSha             :: !(Maybe Text)
+    , whCheckSuitePullRequests         :: !(Vector HookChecksPullRequest)
+    , whCheckSuiteCreatedAt            :: !UTCTime
+    , whCheckSuiteUpdatedAt            :: !UTCTime
+    , whCheckSuiteLatestCheckRunsCount :: !(Maybe Int) -- not included in the check run nested payload
+    , whCheckSuiteCheckRunsUrl         :: !(Maybe URL) -- not included in the check run nested payload
+    , whCheckSuiteHeadCommit           :: !(Maybe HookCheckSuiteCommit) -- not included in the check run nested payload
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookCheckSuite where rnf = genericRnf
+
+-- | Represents the "head_commit" field in the
+--  'CheckSuiteEvent' payload.
+data HookCheckSuiteCommit = HookCheckSuiteCommit
+    { whCheckSuiteCommitSha               :: !Text          -- ^ Sometimes called the commit 'id'.
+    , whCheckSuiteCommitAuthor            :: !HookSimpleUser
+    , whCheckSuiteCommitCommitter         :: !HookSimpleUser
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookCheckSuiteCommit where rnf = genericRnf
+
+-- | Represents the "status" field in the
+--  'HookCheckRun' payload.
+data HookCheckRunStatus
+    -- | Decodes from "queued"
+    = HookCheckRunStatusQueued
+    -- | Decodes from "in_progress"
+    | HookCheckRunStatusInProgress
+    -- | Decodes from "completed"
+    | HookCheckRunStatusCompleted
+    -- | The result of decoding an unknown check run status type
+    | HookCheckRunStatusOther !Text
+    deriving (Eq, Ord, Show, Generic, Typeable, Data)
+
+instance NFData HookCheckRunStatus where rnf = genericRnf
+
+instance FromJSON HookCheckRunStatus where
+  parseJSON = withText "Hook check suite status" $ \t ->
+      case t of
+          "queued"             -> pure HookCheckRunStatusQueued
+          "in_progress"        -> pure HookCheckRunStatusInProgress
+          "completed"          -> pure HookCheckRunStatusCompleted
+          _                    -> pure (HookCheckRunStatusOther t)
+
+-- | Represents the "conclusion" field in the
+--  'HookCheckRun' payload.
+data HookCheckRunConclusion
+    -- | Decodes from "success"
+    = HookCheckRunConclusionSuccess
+    -- | Decodes from "failure"
+    | HookCheckRunConclusionFailure
+    -- | Decodes from "neutral"
+    | HookCheckRunConclusionNeutral
+    -- | Decodes from "cancelled"
+    | HookCheckRunConclusionCancelled
+    -- | Decodes from "timed_out"
+    | HookCheckRunConclusionTimedOut
+    -- | Decodes from "action_required"
+    | HookCheckRunConclusionActionRequired
+    -- | Decodes from "stale"
+    | HookCheckRunConclusionStale
+    -- | The result of decoding an unknown check run conclusion type
+    | HookCheckRunConclusionOther !Text
+    deriving (Eq, Ord, Show, Generic, Typeable, Data)
+
+instance NFData HookCheckRunConclusion where rnf = genericRnf
+
+instance FromJSON HookCheckRunConclusion where
+  parseJSON = withText "Hook check suite status" $ \t ->
+      case t of
+          "success"               -> pure HookCheckRunConclusionSuccess
+          "failure"               -> pure HookCheckRunConclusionFailure
+          "neutral"               -> pure HookCheckRunConclusionNeutral
+          "cancelled"             -> pure HookCheckRunConclusionCancelled
+          "timed_out"             -> pure HookCheckRunConclusionTimedOut
+          "action_required"       -> pure HookCheckRunConclusionActionRequired
+          "stale"                 -> pure HookCheckRunConclusionStale
+          _                       -> pure (HookCheckRunConclusionOther t)
+
+-- FIXME: Missing nested "app", there are examples, but no documentation.
+-- | Represents the "check_run" field in the
+--  'CheckRunEvent' payload.
+data HookCheckRun = HookCheckRun
+    { whCheckRunId                   :: !Int
+    , whCheckRunNodeId               :: !Text
+    , whCheckRunHeadSha              :: !Text
+    , whCheckRunExternalId           :: !Text
+    , whCheckRunUrl                  :: !URL
+    , whCheckRunHtmlUrl              :: !URL
+    , whCheckRunDetailsUrl           :: !URL
+    , whCheckRunStatus               :: !HookCheckRunStatus
+    , whCheckRunConclusion           :: !(Maybe HookCheckRunConclusion)
+    , whCheckRunStartedAt            :: !UTCTime
+    , whCheckRunCompletedAt          :: !(Maybe UTCTime)
+    , whCheckRunOutput               :: !HookCheckRunOutput
+    , whCheckRunName                 :: !Text
+    , whCheckRunCheckSuite           :: !HookCheckSuite
+    , whCheckRunPullRequests         :: !(Vector HookChecksPullRequest)
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookCheckRun where rnf = genericRnf
+
+-- | Represents the "output" field in the
+--  'HookCheckRun' payload.
+data HookCheckRunOutput = HookCheckRunOutput
+    { whCheckRunOutputTitle            :: !(Maybe Text)
+    , whCheckRunOutputSummary          :: !(Maybe Text)
+    , whCheckRunOutputText             :: !(Maybe Text)
+    , whCheckRunOutputAnnotationsCount :: !Int
+    , whCheckRunOutputAnnotationsUrl   :: !URL
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookCheckRunOutput where rnf = genericRnf
+
+-- | Represents the "requested_action" field in the
+--  'CheckRunEvent' payload.
+newtype HookCheckRunRequestedAction = HookCheckRunRequestedAction
+    { whCheckRunRequestedActionIdentifier       :: Text
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookCheckRunRequestedAction where rnf = genericRnf
+
+-- | Represents the "installation" field in the checks payloads.
+data HookChecksInstallation = HookChecksInstallation
+    { whChecksInstallationId     :: Int
+    , whChecksInstallationNodeId :: Text
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookChecksInstallation where rnf = genericRnf
+
+-- | Represents the "pull_requests" field in the checks payloads.
+data HookChecksPullRequest = HookChecksPullRequest
+    { whChecksPullRequestUrl              :: !URL
+    , whChecksPullRequestId               :: !Int
+    , whChecksPullRequestNumber           :: !Int
+    , whChecksPullRequestHead             :: !HookChecksPullRequestTarget
+    , whChecksPullRequestBase             :: !HookChecksPullRequestTarget
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookChecksPullRequest where rnf = genericRnf
+
+-- | Represents the "repo" field in the checks pull_request payloads.
+data HookChecksPullRequestRepository = HookChecksPullRequestRepository
+    { whChecksPullRequestRepositoryId                  :: !Int
+    , whChecksPullRequestRepositoryUrl                 :: !URL
+    , whChecksPullRequestRepositoryName                :: !Text
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookChecksPullRequestRepository where rnf = genericRnf
+
+-- | Represents the repo targets in
+--  the checks pull requests repository payloads.
+data  HookChecksPullRequestTarget = HookChecksPullRequestTarget
+    { whChecksPullRequestTargetSha  :: !Text
+    , whChecksPullRequestTargetRef  :: !Text -- ex "somebranch"
+    , whChecksPullRequestTargetRepo :: !HookChecksPullRequestRepository
+    }
+    deriving (Eq, Show, Typeable, Data, Generic)
+
+instance NFData HookChecksPullRequestTarget where rnf = genericRnf
+
+--- FIXME: Missing nested metadata that provides commit description
+--- FIXME: Missing property "parent" (no examples provided)
 data HookCommit = HookCommit
     { whCommitSha               :: !Text          -- ^ Sometimes called the commit 'id'.
     , whCommitUrl               :: !URL
@@ -426,7 +784,6 @@ data HookCommit = HookCommit
 
 instance NFData HookCommit where rnf = genericRnf
 
-
 -- FIXME: Missing property "assets" (no examples provided)
 data HookRelease = HookRelease
     { whReleaseUrl              :: !URL
@@ -434,6 +791,7 @@ data HookRelease = HookRelease
     , whReleaseUploadUrl        :: !URL
     , whReleaseHtmlUrl          :: !URL
     , whReleaseId               :: !Int
+    , whReleaseNodeId           :: !Text
     , whReleaseTagName          :: !Text
     , whReleaseTargetCommitish  :: !Text
     , whReleaseName             :: !(Maybe Text)
@@ -453,6 +811,7 @@ instance NFData HookRelease where rnf = genericRnf
 data HookPullRequest = HookPullRequest
     { whPullReqUrl              :: !URL
     , whPullReqId               :: !Int
+    , whPullReqNodeId           :: !Text
     , whPullReqHtmlUrl          :: !URL
     , whPullReqDiffUrl          :: !URL
     , whPullReqPatchUrl         :: !URL
@@ -479,7 +838,7 @@ data HookPullRequest = HookPullRequest
     , whPullReqHead             :: !PullRequestTarget
     -- , whPullReqIsMerged         :: !Bool
     -- , whPullReqIsMergeable      :: !Bool
-    -- , whPullReqMergeableState   :: !Text
+    , whPullReqMergeableState   :: !(Maybe Text)              -- ^ Not sent with all events.
     -- , whPullReqMergedBy         :: !(Maybe HookUser)
     , whPullReqCommentCount     :: !(Maybe Int)               -- ^ Not sent with all events.
     , whPullReqRevCommentCount  :: !(Maybe Int)               -- ^ Not sent with all events.
@@ -495,7 +854,7 @@ instance NFData HookPullRequest where rnf = genericRnf
 data PullRequestTarget = PullRequestTarget
     { whPullReqTargetSha :: !Text
     , whPullReqTargetUser :: !HookUser
-    , whPullReqTargetRepo :: !HookRepository
+    , whPullReqTargetRepo :: !(Maybe HookRepository) -- maybe null, see #47
     , whPullReqTargetLabel :: !Text -- ex "user:branch"
     , whPullReqTargetRef :: !Text -- ex "somebranch"
     }
@@ -506,6 +865,7 @@ instance NFData PullRequestTarget where rnf = genericRnf
 -- | Represents the "pull_request" field in the 'PullRequestReviewEvent' payload.
 data HookPullRequestReview = HookPullRequestReview
     { whPullReqReviewId         :: !Int
+    , whPullReqReviewNodeId     :: !Text
     , whPullReqReviewUser       :: !HookUser
     , whPullReqReviewBody       :: !Text
     , whPullReqReviewSubmittedAt :: !UTCTime
@@ -534,6 +894,7 @@ instance NFData HookInstallation where rnf = genericRnf
 data HookDeployment = HookDeployment
     { whDeploymentUrl           :: !URL
     , whDeploymentId            :: !Int
+    , whDeploymentNodeId        :: !Text
     , whDeploymentSha           :: !Text
     , whDeploymentRef           :: !Text
     , whDeploymentTask          :: !Text
@@ -555,6 +916,7 @@ instance NFData HookDeployment where rnf = genericRnf
 data HookDeploymentStatus = HookDeploymentStatus
     { whDeploymentStatusUrl     :: !URL
     , whDeploymentStatusId      :: !Int
+    , whDeploymentStatusNodeId  :: !Text
     , whDeploymentStatusState   :: !Text
     , whDeploymentStatusCreator :: !HookUser
     , whDeploymentStatusDesc    :: !(Maybe Text)
@@ -602,6 +964,7 @@ data HookIssueComment = HookIssueComment
     , whIssueCommentHtmlUrl     :: !URL
     , whIssueCommentIssueUrl    :: !URL
     , whIssueCommentId          :: !Int
+    , whIssueCommentNodeId      :: !Text
     , whIssueCommentUser        :: !HookUser
     , whIssueCommentCreatedAt   :: !UTCTime
     , whIssueCommentUpdatedAt   :: !UTCTime
@@ -616,6 +979,7 @@ data HookCommitComment = HookCommitComment
     { whCommitCommentUrl        :: !URL
     , whCommitCommentHtmlUrl    :: !URL
     , whCommitCommentId         :: !Int
+    , whCommitCommentNodeId     :: !Text
     , whCommitCommentUser       :: !HookUser
     , whCommitCommentPos        :: !(Maybe Int)
     , whCommitCommentLine       :: !(Maybe Int)
@@ -634,6 +998,7 @@ instance NFData HookCommitComment where rnf = genericRnf
 data HookPullRequestReviewComment = HookPullRequestReviewComment
     { whPullReqRevComUrl        :: !URL
     , whPullReqRevComId         :: !Int
+    , whPullReqRevComNodeId     :: !Text
     , whPullReqRevComDiffHunk   :: !Text
     , whPullReqRevComPath       :: !Text
     , whPullReqRevComPos        :: !Int
@@ -662,6 +1027,7 @@ instance FromJSON HookIssue where
       <*> o .: "events_url"
       <*> o .: "html_url"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "number"
       <*> o .: "title"
       <*> o .: "user"
@@ -674,11 +1040,12 @@ instance FromJSON HookIssue where
       <*> o .: "created_at"
       <*> o .: "updated_at"
       <*> o .:? "closed_at"
-      <*> o .: "body"
+      <*> o .:? "body" .!= ""
 
 instance FromJSON HookRepository where
   parseJSON = withObject "HookRepository" $ \o -> HookRepository
       <$> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "name"
       <*> o .: "full_name"
       <*> ((Right <$> o .: "owner") <|> (Left <$> o .: "owner")) -- try complex form first
@@ -723,9 +1090,9 @@ instance FromJSON HookRepository where
       <*> o .: "labels_url"
       <*> o .: "releases_url"
       -- FIXME: Wrap optional number/stringified UTCTime in a helper function? See PushEvent fixture
-      <*> ((o .: "created_at")  <|> (posixSecondsToUTCTime <$> o .: "created_at"))
-      <*> ((o .: "updated_at")  <|> (posixSecondsToUTCTime <$> o .: "updated_at"))
-      <*> ((o .: "pushed_at")   <|> (posixSecondsToUTCTime <$> o .: "pushed_at"))
+      <*> ((o .: "created_at")  <|> (posixSecondsToUTCTime . fromInteger <$> o .: "created_at"))
+      <*> ((o .: "updated_at")  <|> (posixSecondsToUTCTime . fromInteger <$> o .: "updated_at"))
+      <*> ((o .: "pushed_at")   <|> (posixSecondsToUTCTime . fromInteger <$> o .: "pushed_at"))
       <*> o .: "git_url"
       <*> o .: "ssh_url"
       <*> o .: "clone_url"
@@ -747,13 +1114,15 @@ instance FromJSON HookRepository where
 instance FromJSON HookRepositorySimple where
   parseJSON = withObject "HookRepositorySimple" $ \o -> HookRepositorySimple
       <$> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "name"
       <*> o .: "full_name"
       <*> o .: "private"
 
 instance FromJSON HookRepositoryLabel where
   parseJSON = withObject "HookRepositoryLabel" $ \o -> HookRepositoryLabel
-      <$> o .: "url"
+      <$> o .:? "node_id"
+      <*> o .: "url"
       <*> o .: "name"
       <*> o .: "color"
 
@@ -761,6 +1130,7 @@ instance FromJSON HookUser where
   parseJSON = withObject "HookUser" $ \o -> HookUser
       <$> o .: "login"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "avatar_url"
       <*> o .: "gravatar_id"
       <*> o .: "url"
@@ -787,6 +1157,7 @@ instance FromJSON HookOrganization where
   parseJSON = withObject "HookOrganization" $ \o -> HookOrganization
       <$> o .: "login"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "url"
       <*> o .: "repos_url"
       <*> o .: "events_url"
@@ -800,6 +1171,7 @@ instance FromJSON HookOrganization where
 instance FromJSON HookOrganizationInvitation where
   parseJSON = withObject "HookOrganizationInvitation" $ \o -> HookOrganizationInvitation
       <$> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "login"
       <*> o .: "email"
       <*> o .: "role"
@@ -816,18 +1188,50 @@ instance FromJSON HookTeam where
   parseJSON = withObject "HookTeam" $ \o -> HookTeam
       <$> o .: "name"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "slug"
       <*> o .: "permission"
       <*> o .: "url"
       <*> o .: "members_url"
       <*> o .: "repositories_url"
 
+instance FromJSON HookMarketplacePurchase where
+  parseJSON = withObject "HookMarketplacePurchase" $ \o -> HookMarketplacePurchase
+      <$> o .: "account"
+      <*> o .: "billing_cycle"
+      <*> o .: "unit_count"
+      <*> o .: "on_free_trial"
+      <*> (fmap zonedTimeToUTC <$> o .:? "free_trial_ends_on")
+      <*> (fmap zonedTimeToUTC <$> o .:? "next_billing_date")
+      <*> o .: "plan"
+
+instance FromJSON HookMarketplaceAccount where
+  parseJSON = withObject "HookMarketplaceAccount" $ \o -> HookMarketplaceAccount
+      <$> o .: "type"
+      <*> o .: "id"
+      <*> o .: "node_id"
+      <*> o .: "login"
+      <*> o .:? "organization_billing_email"
+
+instance FromJSON HookMarketplacePlan where
+  parseJSON = withObject "HookMarketplacePlan" $ \o -> HookMarketplacePlan
+      <$> o .: "id"
+      <*> o .: "name"
+      <*> o .: "description"
+      <*> o .: "monthly_price_in_cents"
+      <*> o .: "yearly_price_in_cents"
+      <*> o .: "price_model"
+      <*> o .: "has_free_trial"
+      <*> o .: "unit_name"
+      <*> o .: "bullets"
+
 instance FromJSON HookMilestone where
   parseJSON = withObject "HookMilestone" $ \o -> HookMilestone
       <$> o .: "url"
       <*> o .: "html_url"
       <*> o .: "labels_url"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "number"
       <*> o .: "title"
       <*> o .: "description"
@@ -854,6 +1258,7 @@ instance FromJSON HookProject where
       <*> o .: "url"
       <*> o .: "columns_url"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "name"
       <*> o .: "body"
       <*> o .: "number"
@@ -868,6 +1273,7 @@ instance FromJSON HookProjectCard where
       <*> o .: "column_url"
       <*> o .: "column_id"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .:? "note"
       <*> o .: "creator"
       <*> o .: "created_at"
@@ -880,6 +1286,7 @@ instance FromJSON HookProjectColumn where
       <*> o .: "project_url"
       <*> o .: "cards_url"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "name"
       <*> o .: "created_at"
       <*> o .: "updated_at"
@@ -887,11 +1294,91 @@ instance FromJSON HookProjectColumn where
 instance FromJSON HookIssueLabels where
   parseJSON = withObject "HookIssueLabels" $ \o -> HookIssueLabels
       <$> o .:? "id"
+      <*> o .:? "node_id"
       <*> o .: "url"
       <*> o .: "name"
       <*> o .: "color"
       <*> o .:? "default" .!= False
 
+instance FromJSON HookCheckSuite where
+  parseJSON = withObject "HookCheckSuite" $ \o -> HookCheckSuite
+      <$> o .: "id"
+      <*> o .: "node_id"
+      <*> o .:? "head_branch"
+      <*> o .: "head_sha"
+      <*> o .: "status"
+      <*> o .:? "conclusion"
+      <*> o .: "url"
+      <*> o .:? "before"
+      <*> o .:? "after"
+      <*> o .: "pull_requests"
+      <*> o .: "created_at"
+      <*> o .: "updated_at"
+      <*> o .:? "latest_check_runs_count"
+      <*> o .:? "check_runs_url"
+      <*> o .:? "head_commit"
+
+instance FromJSON HookCheckSuiteCommit where
+  parseJSON = withObject "HookCheckSuiteCommit" $ \o -> HookCheckSuiteCommit
+      <$> o .: "id"
+      <*> o .: "author"
+      <*> o .: "committer"
+
+instance FromJSON HookCheckRun where
+  parseJSON = withObject "HookCheckRun" $ \o -> HookCheckRun
+      <$> o .: "id"
+      <*> o .: "node_id"
+      <*> o .: "head_sha"
+      <*> o .: "external_id"
+      <*> o .: "url"
+      <*> o .: "html_url"
+      <*> o .: "details_url"
+      <*> o .: "status"
+      <*> o .:? "conclusion"
+      <*> o .: "started_at"
+      <*> o .:? "completed_at"
+      <*> o .: "output"
+      <*> o .: "name"
+      <*> o .: "check_suite"
+      <*> o .: "pull_requests"
+
+instance FromJSON HookCheckRunOutput where
+  parseJSON = withObject "HookCheckRunOutput" $ \o -> HookCheckRunOutput
+      <$> o .: "title"
+      <*> o .: "summary"
+      <*> o .: "text"
+      <*> o .: "annotations_count"
+      <*> o .: "annotations_url"
+
+instance FromJSON HookCheckRunRequestedAction where
+  parseJSON = withObject "HookCheckRunRequestedAction" $ \o -> HookCheckRunRequestedAction
+      <$> o .: "identifier"
+
+instance FromJSON HookChecksInstallation where
+  parseJSON = withObject "HookChecksInstallation" $ \o -> HookChecksInstallation
+      <$> o .: "id"
+      <*> o .: "node_id"
+
+instance FromJSON HookChecksPullRequest where
+  parseJSON = withObject "HookChecksPullRequest" $ \o -> HookChecksPullRequest
+      <$> o .: "url"
+      <*> o .: "id"
+      <*> o .: "number"
+      <*> o .: "head"
+      <*> o .: "base"
+
+instance FromJSON HookChecksPullRequestTarget where
+    parseJSON = withObject "PullRequestTarget" $ \o -> HookChecksPullRequestTarget
+      <$> o .: "sha"
+      <*> o .: "ref"
+      <*> o .: "repo"
+
+instance FromJSON HookChecksPullRequestRepository where
+  parseJSON = withObject "HookChecksPullRequestRepository" $ \o -> HookChecksPullRequestRepository
+      <$> o .: "id"
+      <*> o .: "url"
+      <*> o .: "name"
+
 instance FromJSON HookCommit where
   parseJSON = withObject "HookCommit" $ \o -> HookCommit
       <$> (o .: "sha" <|> o .: "id")
@@ -908,6 +1395,7 @@ instance FromJSON HookRelease where
       <*> o .: "upload_url"
       <*> o .: "html_url"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "tag_name"
       <*> o .: "target_commitish"
       <*> o .:? "name"
@@ -924,6 +1412,7 @@ instance FromJSON HookPullRequest where
   parseJSON = withObject "HookPullRequest" $ \o -> HookPullRequest
       <$> o .: "url"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "html_url"
       <*> o .: "diff_url"
       <*> o .: "patch_url"
@@ -950,7 +1439,7 @@ instance FromJSON HookPullRequest where
       <*> o .: "head"
       -- <*> o .: "merged"
       -- <*> o .: "mergeable"
-      -- <*> o .: "mergeable_state"
+      <*> o .:? "mergeable_state"
       -- <*> o .: "merged_by"
       <*> o .:? "comments"
       <*> o .:? "review_comments"
@@ -970,6 +1459,7 @@ instance FromJSON PullRequestTarget where
 instance FromJSON HookPullRequestReview where
   parseJSON = withObject "HookPullRequestReview" $ \o -> HookPullRequestReview
       <$> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "user"
       <*> o .: "body"
       <*> o .: "submitted_at"
@@ -989,6 +1479,7 @@ instance FromJSON HookDeployment where
   parseJSON = withObject "HookDeployment" $ \o -> HookDeployment
       <$> o .: "url"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "sha"
       <*> o .: "ref"
       <*> o .: "task"
@@ -1004,6 +1495,7 @@ instance FromJSON HookDeploymentStatus where
   parseJSON = withObject "HookDeploymentStatus" $ \o -> HookDeploymentStatus
       <$> o .: "url"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "state"
       <*> o .: "creator"
       <*> o .:? "description"
@@ -1039,6 +1531,7 @@ instance FromJSON HookIssueComment where
       <*> o .: "html_url"
       <*> o .: "issue_url"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "user"
       <*> o .: "created_at"
       <*> o .: "updated_at"
@@ -1049,6 +1542,7 @@ instance FromJSON HookCommitComment where
       <$> o .: "url"
       <*> o .: "html_url"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "user"
       <*> o .:? "position"
       <*> o .:? "line"
@@ -1062,6 +1556,7 @@ instance FromJSON HookPullRequestReviewComment where
   parseJSON = withObject "HookPullRequestReviewComment" $ \o -> HookPullRequestReviewComment
       <$> o .: "url"
       <*> o .: "id"
+      <*> o .: "node_id"
       <*> o .: "diff_hunk"
       <*> o .: "path"
       <*> o .: "position"
